@@ -1,20 +1,18 @@
 # Debian packaging tools: Custom pretty printer.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: May 28, 2014
+# Last Change: April 19, 2020
 # URL: https://github.com/xolox/python-deb-pkg-tools
 
 """
-Custom pretty printer
-=======================
+Custom pretty printer for parsed control files and package relationships.
 
-The :py:class:`PrettyPrinter` class in the :py:mod:`deb_pkg_tools.print`
-module can be used to pretty print Python expressions containing
-:py:class:`debian.deb822.Deb822` and/or
-:py:class:`deb_pkg_tools.deps.RelationshipSet` objects.
+The :class:`PrettyPrinter` class in the :mod:`deb_pkg_tools.print` module can
+be used to pretty print Python expressions containing :class:`.Deb822` and/or
+:class:`.RelationshipSet` objects.
 
 The custom pretty printer is useful during testing and documenting, for example
-the :py:mod:`doctest` fragments spread throughout the :py:mod:`deb_pkg_tools`
+the :mod:`doctest` fragments spread throughout the :mod:`deb_pkg_tools`
 documentation use the custom pretty printer for human friendly object
 representations.
 """
@@ -23,18 +21,18 @@ representations.
 import pprint
 
 # Modules included in our package.
+from deb_pkg_tools.deb822 import Deb822
 from deb_pkg_tools.deps import RelationshipSet
 
-# External dependencies.
-from debian.deb822 import Deb822
+# Public identifiers that require documentation.
+__all__ = (
+    "CustomPrettyPrinter",
+)
+
 
 class CustomPrettyPrinter(pprint.PrettyPrinter):
 
-    """
-    Custom pretty printer that can be used to pretty print Python
-    expressions containing :py:class:`debian.deb822.Deb822` and/or
-    :py:class:`deb_pkg_tools.deps.RelationshipSet` objects.
-    """
+    """Custom pretty printer for parsed control files and package relationships."""
 
     def _format(self, obj, stream, indent, *args):
         if isinstance(obj, RelationshipSet):
@@ -43,5 +41,3 @@ class CustomPrettyPrinter(pprint.PrettyPrinter):
             pprint.PrettyPrinter._format(self, dict(obj), stream, indent, *args)
         else:
             pprint.PrettyPrinter._format(self, obj, stream, indent, *args)
-
-# vim: ts=4 sw=4 et
