@@ -1,29 +1,28 @@
 # Debian packaging tools: Trivial repository management.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: June 9, 2014
+# Last Change: April 18, 2020
 # URL: https://github.com/xolox/python-deb-pkg-tools
 
 """
-Repository management
-=====================
+Create, update and activate trivial Debian package repositories.
 
-The functions in the :py:mod:`deb_pkg_tools.repo` module make it possible to
+The functions in the :mod:`deb_pkg_tools.repo` module make it possible to
 transform a directory of ``*.deb`` archives into a (temporary) Debian package
 repository:
 
-- :py:func:`update_repository()` creates/updates a `trivial repository`_
+- :func:`update_repository()` creates/updates a `trivial repository`_
 
-- :py:func:`activate_repository()` enables ``apt-get`` to install packages from
+- :func:`activate_repository()` enables ``apt-get`` to install packages from
   the trivial repository
 
-- :py:func:`deactivate_repository()` cleans up after
-  :py:func:`activate_repository()`
+- :func:`deactivate_repository()` cleans up after
+  :func:`activate_repository()`
 
-All of the functions in this module can raise :py:exc:`executor.ExternalCommandFailed`.
+All of the functions in this module can raise :exc:`executor.ExternalCommandFailed`.
 
 You can configure the GPG key(s) used by this module through a configuration
-file, please refer to the documentation of :py:func:`select_gpg_key()`.
+file, please refer to the documentation of :func:`select_gpg_key()`.
 
 .. _trivial repository: http://www.debian.org/doc/manuals/repository-howto/repository-howto#id443677
 """
@@ -34,48 +33,74 @@ import functools
 import glob
 import hashlib
 import logging
+import os
 import os.path
 import pipes
 import re
 import shutil
 import tempfile
 
-# Python 2/3 compatibility.
-try:
-    import ConfigParser as configparser
-except ImportError:
-    import configparser
-
 # External dependencies.
 from executor import execute, ExternalCommandFailed
-from humanfriendly import concatenate, format_path, Spinner, Timer
+from humanfriendly import coerce_boolean, format_path, Timer
+from humanfriendly.decorators import cached
+from humanfriendly.text import concatenate
+from humanfriendly.terminal.spinners import Spinner
+from six.moves import configparser
 
 # Modules included in our package.
 from deb_pkg_tools import config
 from deb_pkg_tools.control import unparse_control_fields
 from deb_pkg_tools.gpg import GPGKey, initialize_gnupg
 from deb_pkg_tools.package import find_package_archives, inspect_package_fields
-from deb_pkg_tools.utils import atomic_lock, optimize_order, sha1
+from deb_pkg_tools.utils import atomic_lock, find_installed_version, optimize_order, sha1
+from deb_pkg_tools.version import Version
+
+# Public identifiers that require documentation.
+__all__ = (
+    "ALLOW_SUDO",
+    "activate_repository",
+    "apt_supports_trusted_option",
+    "deactivate_repository",
+    "get_packages_entry",
+    "load_config",
+    "logger",
+    "scan_packages",
+    "select_gpg_key",
+    "update_repository",
+    "with_repository",
+)
+
+ALLOW_SUDO = coerce_boolean(os.environ.get('DPT_SUDO', 'true'))
+"""
+:data:`True` to enable the use of :man:`sudo` during operations that normally
+require elevated privileges (the default), :data:`False` to disable the use of
+:man:`sudo`. This option is provided for power users to disable the use of
+:man:`sudo` because it may not be available in all build environments. The
+environment variable ``$DPT_SUDO`` can be used to control the value of this
+variable (see :func:`~humanfriendly.coerce_boolean()` for acceptable values).
+"""
 
 # Initialize a logger.
 logger = logging.getLogger(__name__)
 
+
 def scan_packages(repository, packages_file=None, cache=None):
     """
     A reimplementation of the ``dpkg-scanpackages -m`` command in Python.
 
     Updates a ``Packages`` file based on the Debian package archive(s) found in
-    the given directory. Uses :py:class:`.PackageCache` to (optionally) speed
+    the given directory. Uses :class:`.PackageCache` to (optionally) speed
     up the process significantly by caching package metadata and hashes on
-    disk. This explains why this function can be much faster than
-    ``dpkg-scanpackages -m``.
+    disk. This explains why this function can be much faster than the
+    :man:`dpkg-scanpackages` program.
 
     :param repository: The pathname of a directory containing Debian
                        package archives (a string).
     :param packages_file: The pathname of the ``Packages`` file to update
                           (a string). Defaults to the ``Packages`` file in
                           the given directory.
-    :param cache: The :py:class:`.PackageCache` to use (defaults to ``None``).
+    :param cache: The :class:`.PackageCache` to use (defaults to :data:`None`).
     """
     # By default the `Packages' file inside the repository is updated.
     if not packages_file:
@@ -96,27 +121,28 @@ def scan_packages(repository, packages_file=None, cache=None):
     spinner.clear()
     logger.debug("Wrote %i entries to output Packages file in %s.", num_packages, timer)
 
+
 def get_packages_entry(pathname, cache=None):
     """
     Get a dictionary with the control fields required in a ``Packages`` file.
 
     :param pathname: The pathname of the package archive (a string).
-    :param cache: The :py:class:`.PackageCache` to use (defaults to ``None``).
+    :param cache: The :class:`.PackageCache` to use (defaults to :data:`None`).
     :returns: A dictionary with control fields (see below).
 
-    Used by :py:func:`.scan_packages()` to generate ``Packages`` files. The
+    Used by :func:`.scan_packages()` to generate ``Packages`` files. The
     format of ``Packages`` files (part of the Debian binary package repository
     format) is fairly simple:
 
     - All of the fields extracted from a package archive's control file using
-      :py:func:`.inspect_package_fields()` are listed (you have to get these
+      :func:`.inspect_package_fields()` are listed (you have to get these
       fields yourself and combine the dictionaries returned by
-      :py:func:`.inspect_package_fields()` and
-      :py:func:`.get_packages_entry()`);
+      :func:`.inspect_package_fields()` and
+      :func:`.get_packages_entry()`);
 
     - The field ``Filename`` contains the filename of the package archive
       relative to the ``Packages`` file (which is in the same directory in our
-      case, because :py:func:`update_repository()` generates trivial
+      case, because :func:`update_repository()` generates trivial
       repositories);
 
     - The field ``Size`` contains the size of the package archive in bytes;
@@ -124,13 +150,13 @@ def get_packages_entry(pathname, cache=None):
     - The following fields contain package archive checksums:
 
       ``MD5sum``
-        Calculated using :py:func:`hashlib.md5()`.
+        Calculated using the ``md5()`` constructor of the :mod:`hashlib` module.
 
       ``SHA1``
-        Calculated using :py:func:`hashlib.sha1()`.
+        Calculated using the ``sha1()`` constructor of the :mod:`hashlib` module.
 
       ``SHA256``
-        Calculated using :py:func:`hashlib.sha256()`.
+        Calculated using the ``sha256()`` constructor of the :mod:`hashlib` module.
 
     The three checksums are calculated simultaneously by reading the package
     archive once, in blocks of a kilobyte. This is probably why this function
@@ -138,7 +164,10 @@ def get_packages_entry(pathname, cache=None):
     caching).
     """
     if cache:
-        return cache[pathname].package_fields
+        entry = cache.get_entry('package-fields', pathname)
+        value = entry.get_value()
+        if value is not None:
+            return value
     # Prepare to calculate the MD5, SHA1 and SHA256 hashes.
     md5_state = hashlib.md5()
     sha1_state = hashlib.sha1()
@@ -151,33 +180,68 @@ def get_packages_entry(pathname, cache=None):
             sha256_state.update(chunk)
     # Convert the hashes to hexadecimal strings and return the required fields
     # in a dictionary.
-    return dict(Filename=os.path.basename(pathname),
-                Size=str(os.path.getsize(pathname)),
-                MD5sum=md5_state.hexdigest(),
-                SHA1=sha1_state.hexdigest(),
-                SHA256=sha256_state.hexdigest())
+    fields = dict(Filename=os.path.basename(pathname),
+                  Size=str(os.path.getsize(pathname)),
+                  MD5sum=md5_state.hexdigest(),
+                  SHA1=sha1_state.hexdigest(),
+                  SHA256=sha256_state.hexdigest())
+    if cache:
+        entry.set_value(fields)
+    return fields
+
 
 def update_repository(directory, release_fields={}, gpg_key=None, cache=None):
     """
-    Create or update a `trivial repository`_ using the Debian commands
-    ``dpkg-scanpackages`` (reimplemented as :py:class:`scan_packages()`) and
-    ``apt-ftparchive`` (also uses the external programs ``gpg`` and ``gzip``).
-    Raises :py:exc:`.ResourceLockedException` when the given repository
-    directory is being updated by another process.
+    Create or update a `trivial repository`_.
 
     :param directory: The pathname of a directory with ``*.deb`` packages.
     :param release_fields: An optional dictionary with fields to set inside the
                            ``Release`` file.
-    :param gpg_key: The :py:class:`.GPGKey` object used to sign the repository.
-                    Defaults to the result of :py:func:`select_gpg_key()`.
-    :param cache: The :py:class:`.PackageCache` to use (defaults to ``None``).
+    :param gpg_key: The :class:`.GPGKey` object used to sign the repository.
+                    Defaults to the result of :func:`select_gpg_key()`.
+    :param cache: The :class:`.PackageCache` to use (defaults to :data:`None`).
+    :raises: :exc:`.ResourceLockedException` when the given repository
+             directory is being updated by another process.
+
+    This function is based on the Debian programs dpkg-scanpackages_ and
+    apt-ftparchive_ and also uses gpg_ and gzip_. The following files are
+    generated:
+
+    ===============  ==========================================================
+    Filename         Description
+    ===============  ==========================================================
+    ``Packages``     Provides the metadata of all ``*.deb`` packages in the
+                     `trivial repository`_ as a single text file. Generated
+                     using :class:`scan_packages()` (as a faster alternative
+                     to dpkg-scanpackages_).
+    ``Packages.gz``  A compressed version of the package metadata generated
+                     using gzip_.
+    ``Release``      Metadata about the release and hashes of the ``Packages``
+                     and ``Packages.gz`` files. Generated using
+                     apt-ftparchive_.
+    ``Release.gpg``  An ASCII-armored detached GPG signature of the ``Release``
+                     file. Generated using ``gpg --armor --sign
+                     --detach-sign``.
+    ``InRelease``    The contents of the ``Release`` file and its GPG signature
+                     combined into a single human readable file. Generated
+                     using ``gpg --armor --sign --clearsign``.
+    ===============  ==========================================================
+
+    For more details about the ``Release.gpg`` and ``InRelease`` files please
+    refer to the Debian wiki's section on secure-apt_.
+
+    .. _apt-ftparchive: https://manpages.debian.org/apt-ftparchive
+    .. _dpkg-scanpackages: https://manpages.debian.org/dpkg-scanpackages
+    .. _gpg: https://manpages.debian.org/gpg
+    .. _gzip: https://manpages.debian.org/gzip
+    .. _secure-apt: https://wiki.debian.org/SecureApt
     """
     with atomic_lock(directory):
         timer = Timer()
         gpg_key = gpg_key or select_gpg_key(directory)
         # Figure out when the repository contents were last updated.
         contents_last_updated = os.path.getmtime(directory)
-        for archive in find_package_archives(directory):
+        for archive in find_package_archives(directory, cache=cache):
             contents_last_updated = max(contents_last_updated, os.path.getmtime(archive.filename))
         # Figure out when the repository metadata was last updated.
         try:
@@ -187,8 +251,9 @@ def update_repository(directory, release_fields={}, gpg_key=None, cache=None):
             # not cause an unnecessary repository update. That would turn the
             # conditional update into an unconditional update, which is not the
             # intention here :-)
-            if os.path.isfile(os.path.join(directory, 'Release.gpg')) or gpg_key:
-                metadata_files.append('Release.gpg')
+            for signed_file in 'Release.gpg', 'InRelease':
+                if os.path.isfile(os.path.join(directory, signed_file)) or gpg_key:
+                    metadata_files.append(signed_file)
             metadata_last_updated = max(os.path.getmtime(os.path.join(directory, fn)) for fn in metadata_files)
         except Exception:
             metadata_last_updated = 0
@@ -208,7 +273,8 @@ def update_repository(directory, release_fields={}, gpg_key=None, cache=None):
         # 2. If we fail to generate one of the files it's better not to have
         #    changed any of them, for the same reason as point one :-)
         logger.info("%s trivial repository %s ..", "Updating" if metadata_last_updated else "Creating", directory)
-        temporary_directory = tempfile.mkdtemp()
+        temporary_directory = tempfile.mkdtemp(prefix='deb-pkg-tools-', suffix='-update-repo-stage')
+        logger.debug("Using temporary directory: %s", temporary_directory)
         try:
             # Generate the `Packages' file.
             logger.debug("Generating file: %s", format_path(os.path.join(directory, 'Packages')))
@@ -237,20 +303,26 @@ def update_repository(directory, release_fields={}, gpg_key=None, cache=None):
             release_listing = execute(command, capture=True, directory=temporary_directory, logger=logger)
             with open(os.path.join(temporary_directory, 'Release'), 'w') as handle:
                 handle.write(release_listing + '\n')
-            # Generate the `Release.gpg' file by signing the `Release' file with GPG?
+            # Generate the `Release.gpg' and `InRelease' files by signing the `Release' file with GPG?
             gpg_key_file = os.path.join(directory, 'Release.gpg')
+            in_release_file = os.path.join(directory, 'InRelease')
             if gpg_key:
-                logger.debug("Generating file: %s", format_path(gpg_key_file))
                 initialize_gnupg()
+                logger.debug("Generating file: %s", format_path(gpg_key_file))
                 command = "{gpg} --armor --sign --detach-sign --output Release.gpg Release"
                 execute(command.format(gpg=gpg_key.gpg_command), directory=temporary_directory, logger=logger)
-            elif os.path.isfile(gpg_key_file):
+                logger.debug("Generating file: %s", format_path(in_release_file))
+                command = "{gpg} --armor --sign --clearsign --output InRelease Release"
+                execute(command.format(gpg=gpg_key.gpg_command), directory=temporary_directory, logger=logger)
+            else:
                 # XXX If 1) no GPG key was provided, 2) apt doesn't require the
                 # repository to be signed and 3) `Release.gpg' exists from a
                 # previous run, this file should be removed so we don't create an
                 # inconsistent repository index (when `Release' is updated but
                 # `Release.gpg' is not updated the signature becomes invalid).
-                os.unlink(gpg_key_file)
+                for stale_file in gpg_key_file, in_release_file:
+                    if os.path.isfile(stale_file):
+                        os.unlink(stale_file)
             # Move the generated files into the repository directory.
             for entry in os.listdir(temporary_directory):
                 shutil.copy(os.path.join(temporary_directory, entry), os.path.join(directory, entry))
@@ -258,15 +330,19 @@ def update_repository(directory, release_fields={}, gpg_key=None, cache=None):
         finally:
             shutil.rmtree(temporary_directory)
 
+
 def activate_repository(directory, gpg_key=None):
     """
-    Set everything up so that a trivial Debian package repository can be used
-    to install packages without a webserver (this uses the ``file://`` URL
-    scheme to point ``apt-get`` to a directory on the local file system).
+    Activate a local trivial repository.
 
     :param directory: The pathname of a directory with ``*.deb`` packages.
-    :param gpg_key: The :py:class:`.GPGKey` object used to sign the repository.
-                    Defaults to the result of :py:func:`select_gpg_key()`.
+    :param gpg_key: The :class:`.GPGKey` object used to sign the repository.
+                    Defaults to the result of :func:`select_gpg_key()`.
+
+    This function sets everything up so that a trivial Debian package
+    repository can be used to install packages without a webserver. This uses
+    the ``file://`` URL scheme to point :man:`apt-get` to a directory on the
+    local file system.
 
     .. warning:: This function requires ``root`` privileges to:
 
@@ -274,14 +350,16 @@ def activate_repository(directory, gpg_key=None):
                  2. create a ``*.list`` file in ``/etc/apt/sources.list.d`` and
                  3. run ``apt-get update``.
 
-                 This function will use ``sudo`` to gain ``root`` privileges
+                 This function will use :man:`sudo` to gain ``root`` privileges
                  when it's not already running as ``root``.
+
+    .. seealso:: :data:`ALLOW_SUDO`
     """
     directory = os.path.realpath(directory)
     logger.debug("Activating repository: %s", format_path(directory))
     # Generate the `sources.list' file.
     sources_directory = '/etc/apt/sources.list.d'
-    execute('mkdir', '-p', sources_directory, sudo=True, logger=logger)
+    execute('mkdir', '-p', sources_directory, sudo=ALLOW_SUDO, logger=logger)
     sources_file = os.path.join(sources_directory, '%s.list' % sha1(directory))
     logger.debug("Generating file: %s", sources_file)
     sources_entry = ['deb']
@@ -292,22 +370,22 @@ def activate_repository(directory, gpg_key=None):
     command = "echo {text} > {file}"
     execute(command.format(text=pipes.quote(' '.join(sources_entry)),
                            file=pipes.quote(sources_file)),
-            sudo=True, logger=logger)
+            sudo=ALLOW_SUDO, logger=logger)
     # Make apt-get accept the repository signing key?
     gpg_key = gpg_key or select_gpg_key(directory)
     if gpg_key:
         logger.info("Installing GPG key for automatic signing ..")
         initialize_gnupg()
         command = '{gpg} --armor --export | apt-key add -'
-        execute(command.format(gpg=gpg_key.gpg_command), sudo=True, logger=logger)
+        execute(command.format(gpg=gpg_key.gpg_command), sudo=ALLOW_SUDO, logger=logger)
     # Update the package list (make sure it works).
     logger.debug("Updating package list ..")
-    execute("apt-get update", sudo=True, logger=logger)
+    execute("apt-get update", sudo=ALLOW_SUDO, logger=logger)
+
 
 def deactivate_repository(directory):
     """
-    Deactivate a trivial Debian package repository that was previously
-    activated using :py:func:`activate_repository()`.
+    Deactivate a local repository that was previously activated using :func:`activate_repository()`.
 
     :param directory: The pathname of a directory with ``*.deb`` packages.
 
@@ -316,31 +394,40 @@ def deactivate_repository(directory):
                  1. delete a ``*.list`` file in ``/etc/apt/sources.list.d`` and
                  2. run ``apt-get update``.
 
-                 This function will use ``sudo`` to gain ``root`` privileges
+                 This function will use :man:`sudo` to gain ``root`` privileges
                  when it's not already running as ``root``.
+
+    .. seealso:: :data:`ALLOW_SUDO`
     """
     directory = os.path.realpath(directory)
     logger.debug("Deactivating repository: %s", format_path(directory))
     # Remove the `sources.list' file.
     sources_file = os.path.join('/etc/apt/sources.list.d', '%s.list' % sha1(directory))
     logger.debug("Removing file: %s", sources_file)
-    execute('rm', '-f', sources_file, sudo=True, logger=logger)
+    execute('rm', '-f', sources_file, sudo=ALLOW_SUDO, logger=logger)
     # Update the package list (cleanup).
     logger.debug("Updating package list ..")
-    execute("apt-get update", sudo=True, logger=logger)
+    execute("apt-get update", sudo=ALLOW_SUDO, logger=logger)
+
 
 def with_repository(directory, *command, **kw):
     """
-    Create/update a trivial package repository, activate the repository, run an
-    external command (usually ``apt-get install``) and finally deactivate the
-    repository again. Also deactivates the repository when the external command
-    fails and :py:exc:`executor.ExternalCommandFailed` is raised.
+    Execute an external command while a repository is activated.
 
     :param directory: The pathname of a directory containing ``*.deb`` archives
                       (a string).
     :param command: The command to execute (a tuple of strings, passed verbatim
-                    to :py:func:`executor.execute()`).
-    :param cache: The :py:class:`.PackageCache` to use (defaults to ``None``).
+                    to :func:`executor.execute()`).
+    :param cache: The :class:`.PackageCache` to use (defaults to :data:`None`).
+    :raises: :exc:`executor.ExternalCommandFailed` if any external commands fail.
+
+    This function create or updates a trivial package repository, activates the
+    repository, runs an external command (usually ``apt-get install``) and
+    finally deactivates the repository again. Also deactivates the repository
+    when the external command fails and :exc:`executor.ExternalCommandFailed`
+    is raised.
+
+    .. seealso:: :data:`ALLOW_SUDO`
     """
     update_repository(directory=directory,
                       cache=kw.get('cache'))
@@ -350,100 +437,166 @@ def with_repository(directory, *command, **kw):
     finally:
         deactivate_repository(directory)
 
-# Use a global to cache the answer of apt_supports_trusted_option() so we don't
-# execute `dpkg-query --show' and `dpkg --compare-versions' more than once.
-trusted_option_supported = None
 
+@cached
 def apt_supports_trusted_option():
     """
+    Figure out whether apt supports the ``[trusted=yes]`` option.
+
+    :returns: :data:`True` if the option is supported, :data:`False` if it is not.
+
     Since apt version 0.8.16~exp3 the option ``[trusted=yes]`` can be used in a
     ``sources.list`` file to disable GPG key checking (see `Debian bug
     #596498`_). This version of apt is included with Ubuntu 12.04 and later,
     but deb-pkg-tools also has to support older versions of apt. The
-    :py:func:`apt_supports_trusted_option()` function checks if the installed
+    :func:`apt_supports_trusted_option()` function checks if the installed
     version of apt supports the ``[trusted=yes]`` option, so that deb-pkg-tools
     can use it when possible.
 
-    :returns: ``True`` if the option is supported, ``False`` if it is not.
-
     .. _Debian bug #596498: http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=596498
     """
-    global trusted_option_supported
-    if trusted_option_supported is None:
-        try:
-            # Find the installed version of the `apt' package.
-            version = execute('dpkg-query','--show', '--showformat=${Version}', 'apt', capture=True)
-            # Check if the version is >= 0.8.16 (which includes [trusted=yes] support).
-            execute('dpkg','--compare-versions', version, 'ge', '0.8.16~exp3')
-            # If ExternalCommandFailed  is not raised,
-            # `dpkg --compare-versions' reported succes.
-            trusted_option_supported = True
-        except ExternalCommandFailed:
-            trusted_option_supported = False
-    return trusted_option_supported
+    try:
+        # Find the installed version of the `apt' package.
+        apt_version = Version(find_installed_version('apt'))
+        # Check if the version is >= 0.8.16 (which includes [trusted=yes] support).
+        return (apt_version >= Version('0.8.16~exp3'))
+    except ExternalCommandFailed:
+        return False
+
 
 def select_gpg_key(directory):
     """
-    Select a suitable GPG key for repository signing (for use in
-    :py:func:`update_repository()` and :py:func:`activate_repository()`). First
-    the following locations are checked for a configuration file:
+    Select a suitable GPG key for repository signing.
+
+    :param directory: The pathname of the directory that contains the package
+                      repository to sign (a string).
+    :returns: A :class:`.GPGKey` object or :data:`None`.
+
+    Used by :func:`update_repository()` and :func:`activate_repository()` to
+    select the GPG key for repository signing based on a configuration file.
+
+    **Configuration file locations:**
+
+    The following locations are checked for a configuration file:
 
     1. ``~/.deb-pkg-tools/repos.ini``
     2. ``/etc/deb-pkg-tools/repos.ini``
 
-    If both files exist the first one is used. Here is an example configuration
-    with an explicit repository/key pair and a default key:
+    If both files exist only the first one is used.
 
-    .. code-block:: ini
+    **Configuration file contents:**
 
-        [default]
-        public-key-file = ~/.deb-pkg-tools/default.pub
-        secret-key-file = ~/.deb-pkg-tools/default.sec
+    The configuration files are in the ``*.ini`` file format (refer to the
+    :mod:`ConfigParser` module for details). Each section in the configuration
+    file defines a signing key.
 
-        [test]
-        public-key-file = ~/.deb-pkg-tools/test.pub
-        secret-key-file = ~/.deb-pkg-tools/test.sec
-        directory = /tmp
+    The ``directory`` option controls to which directory or directories a
+    signing key applies. The value of this option is the pathname of a
+    directory and supports pattern matching using ``?`` and ``*`` (see the
+    :mod:`fnmatch` module for details).
 
-    Hopefully this is self explanatory: If the repository directory is ``/tmp``
-    the 'test' key pair is used, otherwise the 'default' key pair is used. The
-    'directory' field can contain globbing wildcards like ``?`` and ``*``. Of
-    course you're free to put the actual ``*.pub`` and ``*.sec`` files anywhere
-    you like; that's the point of having them be configurable :-)
+    **The default signing key:**
+
+    If a section does not define a ``directory`` option then that section is
+    used as the default signing key for directories that are not otherwise
+    matched (by a ``directory`` option).
+
+    **Compatibility with GnuPG >= 2.1:**
+
+    `GnuPG 2.1 compatibility`_ was implemented in deb-pkg-tools release 5.0
+    which changes how users are expected to select an isolated GPG key pair:
+
+    - Before deb-pkg-tools 5.0 only GnuPG < 2.1 was supported and the
+      configuration used the ``public-key-file`` and ``secret-key-file``
+      options to configure the pathnames of the public key file and
+      the secret key file:
+
+      .. code-block:: ini
+
+         [old-example]
+         public-key-file = ~/.deb-pkg-tools/default-signing-key.pub
+         secret-key-file = ~/.deb-pkg-tools/default-signing-key.sec
+
+    - In deb-pkg-tools 5.0 support for GnuPG >= 2.1 was added which means the
+      public key and secret key files are no longer configured separately,
+      instead a ``key-store`` option is used to point to a directory in the
+      format of ``~/.gnupg`` containing the key pair:
+
+      .. code-block:: ini
+
+         [new-example]
+         key-store = ~/.deb-pkg-tools/default-signing-key/
+
+      Additionally a ``key-id`` option was added to make it possible to select
+      a specific key pair from a GnuPG profile directory.
+
+    **Staying backwards compatible:**
+
+    By specifying all three of the ``public-key-file``, ``secret-key-file`` and
+    ``key-store`` options it is possible to achieve compatibility with all
+    supported GnuPG versions:
+
+    - When GnuPG >= 2.1 is installed the ``key-store`` option will be used.
+
+    - When GnuPG < 2.1 is installed the ``public-key-file`` and
+      ``secret-key-file`` options will be used.
+
+    In this case the caller is responsible for making sure that a suitable key
+    pair is available in both locations (compatible with the appropriate
+    version of GnuPG).
+
+    **Default behavior:**
 
     If no GPG keys are configured but apt requires local repositories to be
-    signed then this function falls back to selecting an automatically
-    generated signing key. The generated public key and secret key are stored
-    in the directory ``~/.deb-pkg-tools``.
-
-    :param directory: The pathname of the directory that contains the package
-                      repository to sign.
-    :returns: A :py:class:`.GPGKey` object or ``None``.
+    signed (see :func:`apt_supports_trusted_option()`) then this function falls
+    back to selecting an automatically generated signing key. The generated key
+    pair is stored in the directory ``~/.deb-pkg-tools``.
     """
     # Check if the user has configured one or more GPG keys.
     options = load_config(directory)
-    if 'secret-key-file' in options and 'public-key-file' in options:
-        return GPGKey(secret_key_file=os.path.expanduser(options['secret-key-file']),
-                      public_key_file=os.path.expanduser(options['public-key-file']))
+    mapping = [
+        ('key-id', 'key_id'),
+        ('key-store', 'directory'),
+        ('public-key-file', 'public_key_file'),
+        ('secret-key-file', 'secret_key_file'),
+    ]
+    mapped_options = dict(
+        (property_name, options[option_name])
+        for option_name, property_name in mapping
+        if options.get(option_name)
+    )
+    if mapped_options:
+        return GPGKey(**mapped_options)
     if apt_supports_trusted_option():
         # No GPG key was given and no GPG key was configured, however apt
         # supports the [trusted] option so we'll assume the user doesn't care
         # about signing.
-        logger.debug("No GPG key specified but your version of apt doesn't require signing so I'll just skip it :-)")
+        logger.debug("No GPG key specified but your version of apt"
+                     " doesn't require signing so I'll just skip it :-)")
     else:
         # No GPG key was given and no GPG key was configured, but apt doesn't
         # support the [trusted] option so we'll have to sign the repository
         # anyway.
-        logger.debug("No GPG key specified but your version of apt doesn't support the [trusted] option, so I will have to sign the repository anyway ..")
+        logger.debug("No GPG key specified but your version of apt"
+                     " doesn't support the [trusted] option, so I"
+                     " will have to sign the repository anyway ..")
         # XXX About the choice of `user_config_directory' here vs.
         # `system_config_directory': Since we're generating a private
         # key we shouldn't ever store it in a non-secure location.
-        return GPGKey(name="deb-pkg-tools",
-                      description="Automatic signing key for deb-pkg-tools",
-                      secret_key_file=os.path.join(config.user_config_directory, 'automatic-signing-key.sec'),
-                      public_key_file=os.path.join(config.user_config_directory, 'automatic-signing-key.pub'))
+        return GPGKey(
+            # These options are required in order to generate a key pair.
+            name="deb-pkg-tools",
+            description="Automatic signing key for deb-pkg-tools",
+            # These options are required for compatibility with GnuPG < 2.1.
+            public_key_file=os.path.join(config.user_config_directory, 'automatic-signing-key.pub'),
+            secret_key_file=os.path.join(config.user_config_directory, 'automatic-signing-key.sec'),
+            # This option is required for compatibility with GnuPG >= 2.1.
+            directory=os.path.join(config.user_config_directory, 'automatic-signing-key'),
+        )
+
 
 def load_config(repository):
+    """Load repository configuration from a ``repos.ini`` file."""
     repository = os.path.abspath(repository)
     for config_dir in (config.user_config_directory, config.system_config_directory):
         config_file = os.path.join(config_dir, config.repo_config_file)
@@ -460,5 +613,3 @@ def load_config(repository):
                     defaults.update(options)
                     return defaults
     return {}
-
-# vim: ts=4 sw=4 et
